import 'package:flutter/material.dart';
import 'package:fst/fst.dart';

void main() {
  runApp(const CounterApp());
}

class Count extends Kind<int> {
  const Count();
}

const $count = Count();

class Modifier extends Kind<bool> {
  const Modifier();
}

const $modifier = Modifier();

class CounterApp extends StatelessWidget {
  const CounterApp({super.key});

  Widget counter(BuildContext context, StateTreeNode n, Widget child) {
    final (count, setCount) = n.state(0);

    n.action((Increment _) {
      final newCount = count + (n.read($modifier) ? 10 : 1);
      setCount(newCount);
      return newCount;
    });

    n.provide($count, count);

    return child;
  }

  Widget deltaModifier(BuildContext context, StateTreeNode n, Widget child) {
    final (incrementBy10, setIncrementBy10) = n.state(false);
    final focusNode = n.create(() => FocusNode());

    n.provide($modifier, incrementBy10);

    return RawKeyboardListener(
      focusNode: focusNode,
      autofocus: true,
      onKey: (event) {
        if (event.isShiftPressed) {
          setIncrementBy10(true);
        } else {
          setIncrementBy10(false);
        }
      },
      child: child,
    );
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      home: StateTreeBuilder(
        deltaModifier,
        child: StateTreeBuilder(
          counter,
          child: const Home(),
        ),
      ),
    );
  }
}

class Increment extends Intent {
  const Increment();
}

class Home extends StatelessWidget {
  const Home({super.key});

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: const Text('State Tree Example'),
      ),
      body: Center(
        child: StateTreeBuilder((context, n, _) {
          final count = n.consume($count);
          return Text('Count: $count');
        }),
      ),
      floatingActionButton: FloatingActionButton(
        onPressed: () {
          final result = Actions.invoke(context, const Increment());

          if (result is int && result % 10 == 0) {
            ScaffoldMessenger.of(context).showSnackBar(
              const SnackBar(
                content: Text('Count is a multiple of 10!'),
              ),
            );
          }
        },
        child: const Icon(Icons.add),
      ),
    );
  }
}
